import asyncio
import re
import json
import logging
import yaml
from pathlib import Path
from typing import Any, Callable
import weave
from gpt_translate.configs import EvalConfig
from gpt_translate.loader import MDPage
from gpt_translate.utils import openai_client
from gpt_translate.prompts import PromptTemplate


@weave.op
def validate_links(original_page: MDPage, translated_page: MDPage, model_output: Any) -> dict:
    """
    Validate that the links in the original page are the same as the links in the translated page.
    """
    original_links = original_page.links
    translated_links = translated_page.links

    missing_links = [link for link in original_links if link not in translated_links]
    extra_links = [link for link in translated_links if link not in original_links]
    return dict(
        links_match=len(missing_links) == 0 and len(extra_links) == 0,
        missing_links=[l.target for l in missing_links],
        extra_links=[l.target for l in extra_links],
        total_links=len(original_links),
    )


@weave.op
def validate_headers(original_page: MDPage, translated_page: MDPage, model_output: Any) -> dict:
    """
    Validate that the headers in the original page are the same as the headers in the translated page.
    """
    original_header = original_page.header
    translated_header = translated_page.header
    title_match = original_header.title == translated_header.title
    description_match = original_header.description == translated_header.description
    slug_match = original_header.slug == translated_header.slug
    displayed_sidebar_match = (
        original_header.displayed_sidebar == translated_header.displayed_sidebar
    )
    imports_match = original_header.imports == translated_header.imports
    return dict(
        title_match=title_match,
        description_match=description_match,
        slug_match=slug_match,
        displayed_sidebar_match=displayed_sidebar_match,
        imports_match=imports_match,
    )


@weave.op
def validate_tabs(translated_page: MDPage, model_output: Any) -> bool:
    """
    Validate the Tabs in the docosaurus format
    """
    tab_pattern = re.compile(
        r"""
        <Tabs\s*[^>]*>\s*
        (?:<TabItem\s*[^>]*>\s*.*?\s*</TabItem>\s*)+
        </Tabs>
        """,
        re.DOTALL | re.VERBOSE,
    )

    results = bool(tab_pattern.search(translated_page.content))

    return {"tabs_format_valid": results}


def _validate_technical_words(original_page: MDPage, translated_page: MDPage, dictionary: dict) -> dict:
    """Validate that technical words are translated correctly and maintain their frequency."""
    if not dictionary:
        return {"tech_words_percentage": 1.0, "mismatched": []}
    
    original_word_count = sum(original_page.content.count(re.escape(k)) for k in dictionary)
    translated_word_count = 0
    mismatched = []

    for k, v in dictionary.items():
        original_count = original_page.content.count(re.escape(k))
        translated_count = translated_page.content.count(re.escape(v))
        translated_word_count += translated_count

        if original_count != translated_count:
            mismatched.append((k, v))
    
    tech_words_percentage = translated_word_count / original_word_count if original_word_count > 0 else 1.0
    
    return {
        "tech_words_percentage": tech_words_percentage,
        "mismatched": mismatched
    }

def validate_technical_words(dictionary: dict) -> Callable:
    @weave.op(name="validate_technical_words")
    def _inner(original_page: MDPage, translated_page: MDPage, model_output: Any) -> dict:
        return _validate_technical_words(original_page, translated_page, dictionary)
    return _inner

ALL_SCORERS = [
    validate_links,
    validate_headers,
    validate_tabs,
]

class LLMJudge(weave.Model):
    system_prompt: str
    evaluation_prompt: str
    model_args: dict

    @weave.op
    async def predict(self, original_page: MDPage, translated_page: MDPage):
        """Evaluate the translation"""
        messages = [
            {"role": "system", "content": self.system_prompt},
            {
                "role": "user",
                "content": self.evaluation_prompt.format(
                    original_page=original_page.content,
                    translated_page=translated_page.content,
                ),
            },
        ]
        res = await openai_client.chat.completions.create(
            messages=messages,
            **self.model_args,
            response_format={"type": "json_object"},
        )
        extracted = res.choices[0].message.content
        analysis = json.loads(extracted)
        return {"analysis": analysis, "translated_page": translated_page}



class Evaluator:
    def __init__(self, config: EvalConfig, scorers: list[Callable] = ALL_SCORERS):
        self.config = config
        self.dataset = self.get_dataset()
        self.prompt_template = self._load_prompts()
        self.scorers = self.setup_scorers(scorers)
        self.model_args = {
            "model": config.model,
            "temperature": config.temperature,
            "max_tokens": config.max_tokens,
        }
        self.setup_judge()

    def setup_scorers(self, scorers: list[Callable]):
        dictionary = yaml.safe_load(self.prompt_template.dictionary)
        return scorers + [validate_technical_words(dictionary)]

    def _load_prompts(self):
        config_folder = Path(self.config.config_folder)
        language = self.config.language
        return PromptTemplate.from_folder(config_folder, language)

    def setup_judge(self):
        self.judge = LLMJudge(
            system_prompt=self.prompt_template.system_prompt,
            evaluation_prompt=self.prompt_template.evaluation_prompt,
            model_args=self.model_args,
        )

    def get_dataset(self):
        logging.info(f"Getting dataset: {self.config.eval_dataset}")
        dataset = weave.ref(self.config.eval_dataset).get()

        def deserialize(md_page):
            """md_page is a string of a dictionary"""
            return MDPage.model_validate(json.loads(md_page))

        ds = [
            {
                "original_page": deserialize(row["original_page"]),
                "translated_page": deserialize(row["translated_page"]),
            }
            for row in dataset.rows
        ]
        logging.info(f"Running eval on {len(ds)} pages")
        return ds

    def evaluate(self):
        evaluation = weave.Evaluation(
            dataset=self.dataset,
            scorers=self.scorers,
        )
        asyncio.run(evaluation.evaluate(self.judge))



import weave
import asyncio
import logging
from pathlib import Path

from rich.logging import RichHandler
import simple_parsing
from dataclasses import dataclass

from gpt_translate.translate import _translate_file, _translate_files
from gpt_translate.utils import get_md_files, _copy_images, get_modified_files
from gpt_translate.configs import EvalConfig, setup_parsing, DEFAULT_EVAL_CONFIG_PATH, CopyImagesArgs, NewFilesArgs
from gpt_translate.evaluate import Evaluator



def setup_logging(debug=False, silence_openai=True, weave_project=None):
    """Setup logging"""
    # Initialize weave
    if weave_project:
        weave.init(weave_project)

    # Setup rich logger
    level = "DEBUG" if debug else "INFO"
    logging.basicConfig(
        level=level, format="%(message)s", datefmt="[%X]", handlers=[RichHandler()]
    )

    # silence openai logger
    if silence_openai:
        logging.getLogger("httpcore").setLevel(logging.WARNING)
        logging.getLogger("openai").setLevel(logging.WARNING)
        logging.getLogger("httpx").setLevel(logging.WARNING)


def translate_file(args=None):
    # logs_args, translation_args, file_args, model_args = setup_parsing(args=args)
    config = setup_parsing(args=args)
    setup_logging(
        config.debug,
        silence_openai=config.silence_openai,
        weave_project=config.weave_project,
    )
    logging.info(f"{config.dumps_yaml()}")
    asyncio.run(
        _translate_file(
            input_file=config.input_file,
            out_file=config.out_file,
            replace=config.replace,
            language=config.language,
            config_folder=config.config_folder,
            remove_comments=config.remove_comments,
            do_translate_header_description=config.do_translate_header_description,
            model_args={
                "model": config.model,
                "temperature": config.temperature,
                "max_tokens": config.max_tokens,
            },
        )
    )


def translate_files(args=None):
    config = setup_parsing(args=args)
    setup_logging(
        config.debug,
        silence_openai=config.silence_openai,
        weave_project=config.weave_project,
    )
    logging.info(f"{config.dumps_yaml()}")
    asyncio.run(
        _translate_files(
            input_files=config.input_file,
            input_folder=config.input_folder,
            out_folder=config.out_folder,
            replace=config.replace,
            language=config.language,
            config_folder=config.config_folder,
            remove_comments=config.remove_comments,
            do_translate_header_description=config.do_translate_header_description,
            max_openai_concurrent_calls=config.max_openai_concurrent_calls,
            model_args={
                "model": config.model,
                "temperature": config.temperature,
                "max_tokens": config.max_tokens,
            },
        )
    )


def translate_folder(args=None):
    config = setup_parsing(args=args)
    setup_logging(
        config.debug,
        silence_openai=config.silence_openai,
        weave_project=config.weave_project,
    )
    logging.info(f"{config.dumps_yaml()}")
    input_files = get_md_files(config.input_folder)[: config.limit]
    asyncio.run(
        _translate_files(
            input_files=input_files,
            input_folder=config.input_folder,
            out_folder=config.out_folder,
            replace=config.replace,
            language=config.language,
            config_folder=config.config_folder,
            remove_comments=config.remove_comments,
            do_translate_header_description=config.do_translate_header_description,
            max_openai_concurrent_calls=config.max_openai_concurrent_calls,
            model_args={
                "model": config.model,
                "temperature": config.temperature,
                "max_tokens": config.max_tokens,
            },
        )
    )


def eval(args=None):
    config = setup_parsing(args=args, config_class=EvalConfig, config_path=DEFAULT_EVAL_CONFIG_PATH)
    setup_logging(
        config.debug,
        silence_openai=config.silence_openai,
        weave_project=config.weave_project,
    )
    logging.info(f"{config.dumps_yaml()}")

    evaluator = Evaluator(config)
    evaluator.evaluate()


def copy_images(args=None):
    args = simple_parsing.parse(args=args, config_class=CopyImagesArgs)
    print(args)
    _copy_images(args.src_path, args.dst_path)


def new_files(args=None):
    args = simple_parsing.parse(args=args, config_class=NewFilesArgs)
    print(args)
    setup_logging(debug=False)
    modified_files = get_modified_files(
        repo_path=args.repo, extension=args.extension, since_days=args.since_days
    )
    with open(args.out_file, "w") as f:
        for file in modified_files:
            f.write(str(file) + "\n")
            print(file)



from gpt_translate.utils import file_is_empty, get_md_files


def test_empty_file():
    "Check that the files are non empty"
    assert file_is_empty("tests/data/empty.md")
    assert not file_is_empty("tests/data/intro.md")


def test_get_md_files():
    "Check that we get the right files"
    files = get_md_files("tests/data")
    assert len(files) == 2
    assert files[0].name == "empty.md"

    # Check that we can get a single file
    file = get_md_files("tests/data/intro.md")
    assert len(file) == 1

    # Check that we can get a single file with a glob
    none_files = get_md_files("tests/data", files_glob="*.txt")
    assert len(none_files) == 0

    # Check that we can get a single file with a regex
    file_re = get_md_files("tests/data", file_re="intro")
    assert len(file_re) == 1



import logging
import asyncio
from pathlib import Path
from dataclasses import dataclass
from tqdm.asyncio import tqdm

import weave
from pydantic import model_validator, Field

from gpt_translate.prompts import PromptTemplate
from gpt_translate.loader import (
    remove_markdown_comments,
    MDPage,
    Header,
)
from gpt_translate.utils import (
    file_is_empty,
    count_tokens,
    longer_create,
    to_weave_dataset,
)


## Globals
REPLACE = False
REMOVE_COMMENTS = True
MAX_OPENAI_CONCURRENT_CALLS = 7  # Adjust the limit as needed


@dataclass
class TranslationResult:
    content: str
    tokens: int


@weave.op
async def translate_content(md_content: str, prompt: PromptTemplate, **model_args):
    """Translate a markdown chunk asynchronously
    md_content: markdown content
    prompt: PromptTemplate object
    return: translated page
    """
    output = await longer_create(
        messages=prompt.format(md_chunk=md_content), **model_args
    )
    return TranslationResult(content=output, tokens=count_tokens(output))


class Translator(weave.Object):
    "A class to translate markdown files asynchronously"
    config_folder: Path
    language: str = "ja"
    do_translate_header_description: bool = True
    model_args: dict = dict(model="gpt-4o", temperature=1.0)
    prompt_template: PromptTemplate = Field(default=None)

    @model_validator(mode="before")
    def initialize_fields(cls, values):
        config_folder = Path(values.get("config_folder"))
        language = values.get("language", "ja")
        prompt_template = PromptTemplate.from_folder(config_folder, language)
        values.update(
            {
                "config_folder": config_folder,
                "prompt_template": prompt_template,
            }
        )
        return values

    @weave.op
    async def translate_file(self, md_file: str, remove_comments: bool = True):
        """Translate a markdown file asynchronously"""
        with open(md_file, "r") as f:
            md_content = f.read()
        if remove_comments:
            logging.debug("Removing comments")
            md_content = remove_markdown_comments(md_content)
        if len(md_content.strip()) < 20:
            logging.warning(f"File may be empty: {md_file}")
        md_page = MDPage.from_raw_content(filename=md_file, raw_content=md_content)
        logging.debug(
            f"[bold red blink]Calling OpenAI [/bold red blink]with {self.model_args}\nFile: {md_file}\nContent:\n{md_content[:100]}...",
            extra={"markup": True},
        )
        translated_page = await self.translate_page(md_page)
        return {"original_page": md_page, "translated_page": translated_page}

    @weave.op
    async def translate_page(self, md_page: MDPage, translate_header: bool = True):
        """Translate a markdown page asynchronously"""
        # chunks = split_markdown(md_page.content)
        # translated_content = await self.translate_splitted_md(chunks)
        translated_content = await translate_content(
            md_page.content, self.prompt_template, **self.model_args
        )
        logging.debug(f"Translated content: {translated_content}")
        if (
            md_page.header.description and self.do_translate_header_description
        ):  # check if header contains a description
            translated_header_description = await self.translate_header_description(
                md_page
            )
            logging.debug(
                f"Translating header description: {md_page.header.description}"
            )
            new_header = Header(
                title=md_page.header.title,
                description=(
                    translated_header_description.content
                    if md_page.header.description
                    else None
                ),
                slug=md_page.header.slug,
                displayed_sidebar=md_page.header.displayed_sidebar,
                imports=md_page.header.imports,
            )
        else:
            new_header = md_page.header
        return MDPage(
            filename=md_page.filename,
            content=translated_content.content,
            header=new_header,
        )

    @weave.op
    async def translate_header_description(self, md_page: MDPage):
        """Translate the header description"""
        translated_description = await translate_content(
            md_page.header.description, self.prompt_template, **self.model_args
        )
        return translated_description


@weave.op
async def _translate_file(
    input_file: str,  # File to translate
    out_file: str,  # File to save the translated file to
    replace: bool = REPLACE,  # Replace existing file
    language: str = "es",  # Language to translate to
    config_folder: str = "./configs",  # Config folder
    remove_comments: bool = REMOVE_COMMENTS,  # Remove comments
    do_translate_header_description: bool = True,  # Translate the header description
    model_args: dict = dict(model="gpt-4o", temperature=1.0),  # model args
) -> MDPage:
    """Translate a markdown file asynchronously"""
    if file_is_empty(input_file):
        raise ValueError(f"File {input_file} is empty")

    # check it is a md file
    if Path(input_file).suffix not in [".md", ".mdx"]:
        raise ValueError(f"File {input_file} is not a markdown file")
    out_file = Path(out_file)
    if out_file.exists() and not replace and not file_is_empty(out_file):
        logging.info(f"File {out_file} already exists. Use --replace to overwrite.")
    else:
        out_file.parent.mkdir(parents=True, exist_ok=True)
        try:
            translator = Translator(
                config_folder=config_folder,
                language=language,
                do_translate_header_description=do_translate_header_description,
                model_args=model_args,
            )
            translation_results = await translator.translate_file(
                input_file, remove_comments
            )
            with open(out_file, "w", encoding="utf-8") as f:
                f.write(str(translation_results["translated_page"]))
            logging.info(
                f"✅ Translated file saved to [green]{out_file}[/green]",
                extra={"markup": True},
            )
            return translation_results
        except Exception as e:
            logging.error(f"❌ Error translating {input_file}: {e}")
            raise e


@weave.op
async def _translate_files(
    input_files: list[str],  # Files to translate
    input_folder: str,  # folder where the file lives
    out_folder: str,  # Folder to save the translated files to
    replace: bool = REPLACE,  # Replace existing file
    language: str = "es",  # Language to translate to
    config_folder: str = "./configs",  # Config folder
    remove_comments: bool = REMOVE_COMMENTS,  # Remove comments
    do_translate_header_description: bool = True,  # Translate the header description
    model_args: dict = dict(model="gpt-4o", temperature=1.0),  # model args
    max_openai_concurrent_calls: int = MAX_OPENAI_CONCURRENT_CALLS,  # Maximum number of concurrent calls to OpenAI
):
    # let's make input_files support a txt file with a list of files
    if not isinstance(input_files, list):
        if Path(input_files).suffix == ".txt":
            logging.info(f"Reading {input_files}")
        input_files = Path(input_files).read_text().splitlines()
    input_files = [
        Path(f)
        for f in input_files
        if (Path(f).suffix in [".md", ".mdx"] and Path(f).exists())
    ]
    logging.info(
        f"Translating {len(input_files)} file" + ("s" if len(input_files) > 1 else "")
    )
    input_files.sort()
    input_folder = Path(input_folder)
    out_folder = Path(out_folder)
    if not input_folder.is_dir():
        raise ValueError(f"{input_folder} is not a folder")

    semaphore = asyncio.Semaphore(max_openai_concurrent_calls)

    async def _translate_with_semaphore(md_file):
        async with semaphore:
            out_file = out_folder / md_file.relative_to(input_folder)
            translation_results = await _translate_file(
                input_file=str(md_file),
                out_file=str(out_file),
                replace=replace,
                language=language,
                config_folder=config_folder,
                remove_comments=remove_comments,
                do_translate_header_description=do_translate_header_description,
                model_args=model_args,
            )
            translation_results.update(
                {
                    "input_file": str(md_file),
                    "output_file": str(out_file),
                    "language": language,
                }
            )
            return translation_results

    tasks = [_translate_with_semaphore(md_file) for md_file in input_files]

    results = await tqdm.gather(*tasks, desc="Translating files")

    dataset = to_weave_dataset(name=f"Translation-{language}", rows=results)
    weave.publish(dataset)


if __name__ == "__main__":
    from gpt_translate.cli import setup_logging

    setup_logging(debug=True, silence_openai=True, weave_project="gpt-translate")
    asyncio.run(
        _translate_files(
            input_files="one.txt",
            input_folder="../docodile/docs_main/",
            out_folder="../docodile/docs/",
            replace=True,
            language="ja",
            config_folder="./configs",
        )
    )



# This file is in sync with the config.yaml file

from dataclasses import dataclass
from pathlib import Path
import simple_parsing
from simple_parsing.helpers import Serializable

DEFAULT_CONFIG_PATH = "./configs/config.yaml"
DEFAULT_EVAL_CONFIG_PATH = "./configs/eval_config.yaml"


@dataclass
class TranslateConfig(Serializable):
    model: str  # Model to use
    temperature: float  # Temperature to use
    max_tokens: int  # Max tokens to use

    debug: bool  # Debug mode
    weave_project: str  # Weave project
    silence_openai: bool  # Silence OpenAI logger

    language: str  # Language to translate to
    config_folder: str  # Config folder
    replace: bool  # Replace existing file
    remove_comments: bool  # Remove comments
    do_translate_header_description: bool  # Translate the header description
    max_openai_concurrent_calls: int  # Max number of concurrent calls to OpenAI

    input_file: str  # File to translate, can be a .txt file with a list of files when used with translate.files
    out_file: str  # File to save the translated file to
    input_folder: str = "./docs/"  # Folder to translate
    out_folder: str = "./docs_translated/"  # Folder to save the translated files to
    limit: int = None  # Limit number of files to translate

@dataclass
class EvalConfig(Serializable):
    model: str  # Model to use
    temperature: float  # Temperature to use
    max_tokens: int  # Max tokens to use

    debug: bool  # Debug mode
    weave_project: str  # Weave project
    silence_openai: bool  # Silence OpenAI logger

    language: str  # Language to translate to
    config_folder: str  # Config folder
    max_openai_concurrent_calls: int  # Max number of concurrent calls to OpenAI

    eval_dataset: str = "Translation-ja:latest"  # the Weave dataset name to evaluate

@dataclass
class CopyImagesArgs:
    src_path: Path
    dst_path: Path

@dataclass
class NewFilesArgs:
    repo: Path
    extension: str = ".md"
    since_days: int = 14
    out_file: Path = "./changed_files.txt"


def setup_parsing(args=None, config_class=TranslateConfig, config_path=DEFAULT_CONFIG_PATH):
    config = simple_parsing.parse(
        config_class=config_class,
        config_path=config_path,
        args=args,
        add_config_path_arg=True,
    )
    return config



from dataclasses import dataclass
from gpt_translate.evaluate import validate_tabs

@dataclass
class MDPage:
    content: str


def test_validate_tabs_valid():
    content = """
    <Tabs>
        <TabItem value="python" label="Python">
            Python content
        </TabItem>
        <TabItem value="javascript" label="JavaScript">
            JavaScript content
        </TabItem>
    </Tabs>
    """
    page = MDPage(content)
    assert validate_tabs(page, model_output=None)["tabs_format_valid"] is True


def test_validate_tabs_invalid():
    content = """
    <Tabs>
        <TabItem value="python" label="Python">
            Python content
        </TabItem>
        <TabItem value="javascript" label="JavaScript">
            JavaScript content
    </Tabs>
    """
    page = MDPage(content)
    assert validate_tabs(page, model_output=None)["tabs_format_valid"] is False


def test_validate_tabs_format_with_random_text():
    content = """
    Some random text before the tabs.
    <Tabs>
        <TabItem value="python" label="Python">
            Python content
        </TabItem>
        <TabItem value="javascript" label="JavaScript">
            JavaScript content
        </TabItem>
    </Tabs>
    Some random text after the tabs.
    """
    page = MDPage(content)
    assert validate_tabs(page, model_output=None)["tabs_format_valid"] is True



import time
import git
import logging
import weave
from datetime import datetime, timedelta
from pathlib import Path

import pydantic
import tiktoken
from openai import AsyncOpenAI
from fastcore.xtras import globtastic

# Use the OpenAI API in async mode
try:
    openai_client = AsyncOpenAI()
except Exception:
    logging.warning("Failed to initialize OpenAI client. Using a dummy client.")
    openai_client = None

MODEL = "gpt-4"


@weave.op
async def longer_create(messages=None, max_tokens=4096, **kwargs):
    """
    longer_create is a function that extends the max_tokens beyond the default 4096 by recursively calling the create method if the finish_reason is hitting the max_tokens.
    """
    if messages is None:
        messages = []

    res = await openai_client.chat.completions.create(
        messages=messages, max_tokens=max_tokens, **kwargs
    )
    message_content = res.choices[0].message.content
    logging.debug(res.usage)
    logging.debug(
        f"[blue]OpenAI response:\n{message_content[:100]}...[/blue]",
        extra={"markup": True},
    )

    finish_reason = res.choices[0].finish_reason
    if finish_reason == "length":
        # trim message to the last separator
        process_tail = remove_after(message_content)
        messages.append({"role": "assistant", "content": process_tail["text"]})
        # Recursively call the function with the last assistant's message
        logging.debug(f"Recursively calling with {messages[-1]['content'][:100]}")
        next_response = await longer_create(
            messages=messages, max_tokens=max_tokens, **kwargs
        )
        return process_tail["text"] + next_response
    else:
        return message_content


@weave.op
def remove_after(text, sep=["\n\n", "\n", ". ", ", "]):
    "Find the last `sep` from the end of the string backwards. Remove the trailing text after the line break."
    index = None
    output_text = text
    for s in sep:
        if s in text:
            index = text.rfind(s)
            output_text = text[: (index + len(s))]
            break
    else:
        s = None
    return {"text": output_text, "sep_break": s}


def measure_execution_time(func):
    def wrapper(*args, **kwargs):
        start_time = time.perf_counter()
        result = func(*args, **kwargs)
        end_time = time.perf_counter()
        elapsed_time = end_time - start_time
        logging.debug(
            f"Function {func.__name__} took {elapsed_time:.4f} seconds to execute."
        )
        return result

    return wrapper


def file_is_empty(file: str | Path):
    return not Path(file).read_text().strip()


def get_md_files(path: Path | str, files_glob: str = "*.md", file_re: str = None):
    """
    Get a list of markdown files in the given path.
    """
    path = Path(path)
    if path.is_file():
        return [path]
    files = globtastic(path, file_glob=files_glob, file_re=file_re)
    return [Path(f) for f in files.sorted() if Path(f).exists()]


def _copy_images(src_path: Path | str, dst_path: Path | str):
    "recursively copy images from src_path to dst_path"
    src_path = Path(src_path)
    dst_path = Path(dst_path)

    # Define image file extensions
    image_extensions = [".png", ".jpg", ".jpeg", ".gif", ".bmp", ".svg"]

    for src_file in src_path.rglob("*"):
        if src_file.suffix.lower() in image_extensions:
            # Compute relative path
            rel_path = src_file.relative_to(src_path)
            # Construct destination path
            dst_file = dst_path / rel_path

            # Create parent directories if they don't exist
            dst_file.parent.mkdir(parents=True, exist_ok=True)

            # Copy the file
            dst_file.write_bytes(src_file.read_bytes())
            logging.debug(f"Copied {src_file} to {dst_file}")


def count_tokens(chunk, model=MODEL):
    "Count the number of tokens in a chunk"
    enc = tiktoken.encoding_for_model(model)
    return len(enc.encode(chunk))


def concat_md_chunks(chunks, sep="\n\n"):
    return sep.join(chunks)


def get_modified_files(repo_path: Path, since_days: int = 14, extension: str = ".md"):
    """
    Get a list of modified files in the last `since_days` days.
    """
    path = Path(repo_path).resolve().absolute()
    if not path.is_dir():
        path = path.parent
    logging.info(
        f"Searching for modified files in the last {since_days} days in: {path}"
    )
    # Open the repository
    repo = git.Repo(repo_path, search_parent_directories=True)
    repo_path = Path(repo.git_dir).parent  # this is the .git file

    logging.info(f"Repo: {repo.git_dir}")
    # Calculate the date since which to look for commits
    since_date = datetime.now() - timedelta(days=since_days)

    # Get commits since the specified date
    commits = list(repo.iter_commits(since=since_date.isoformat()))

    # Use a set to store unique modified files
    modified_files = set()

    # Iterate over the commits and collect modified files
    for commit in commits:
        for diff in commit.diff(None, create_patch=False):
            if diff.a_path:
                modified_files.add(repo_path / Path(diff.a_path))
            if diff.b_path:
                modified_files.add(repo_path / Path(diff.b_path))
    modified_files = list(
        [m for m in modified_files if m.suffix == extension and m.is_relative_to(path)]
    )
    modified_files = [f.relative_to(repo_path) for f in modified_files]
    logging.info(
        f"Found {len(modified_files)} modified files in the last {since_days} days in: {repo_path}"
    )
    return modified_files


def to_weave_dataset(name: str, rows: list) -> weave.Dataset:
    # serialize the pydantic objects that are in the dictionary:
    for result in rows:
        for k, v in result.items():
            # check if we have weave.Object or pydantic.BaseModel
            if isinstance(v, pydantic.BaseModel | weave.Object):
                result[k] = v.model_dump_json()

    # push to weave
    dataset = weave.Dataset(name=name, description="Translation files", rows=rows)
    return dataset



from gpt_translate.loader import (
    remove_markdown_comments,
    split_markdown,
    Header,
    extract_header,
)


def test_remove_markdown_comments():
    # Example usage with your Markdown content
    markdown_content = """
Some content
<!-- This is a comment
It spans multiple lines -->
More content
"""
    cleaned_content = remove_markdown_comments(markdown_content)

    assert (
        cleaned_content
        == """
Some content


More content
"""
    )


def test_split_markdown():
    # Example usage with your Markdown content
    markdown_content = """
Some pre-header content
# Header 1
Content under header 1
## Header 1.1
Content under header 1.1
# Header 2
Content under header 2
"""

    chunks = split_markdown(markdown_content)

    # no line breaks
    assert (
        chunks[2]
        == """## Header 1.1
Content under header 1.1"""
    )


def test_simple_header():
    header = """---
description: description
slug: /guides/app
displayed_sidebar: default
---"""
    content = """
# title
"""
    page = header + "\n" + content
    extracted_header = extract_header(page)["header"]
    assert extracted_header == header
    header_obj = Header.from_string(extracted_header)
    assert header_obj.description == "description"
    assert header_obj.displayed_sidebar == "default"
    assert header_obj.slug == "/guides/app"
    assert str(header_obj) == header


def test_header_with_imports():
    header = """import 1;
import 2;
import 3;"""
    content = """
# title
"""
    page = header + "\n" + content
    extracted_header = extract_header(page)["header"]
    assert extracted_header == header
    header_obj = Header.from_string(extracted_header)
    assert header_obj.description is None
    assert header_obj.displayed_sidebar is None
    assert header_obj.slug is None
    assert header_obj.imports == "import 1;\nimport 2;\nimport 3;"
    assert str(header_obj) == header


def test_header_with_frontmatter_and_imports():
    header = """---
description: description
slug: /guides/app
displayed_sidebar: default
---
import 1;
import 2;
import 3;"""
    content = """
# title
"""
    page = header + "\n" + content
    extracted_header = extract_header(page)["header"]
    assert extracted_header == header
    header_obj = Header.from_string(extracted_header)
    assert header_obj.description == "description"
    assert header_obj.displayed_sidebar == "default"
    assert header_obj.slug == "/guides/app"
    assert header_obj.imports == "import 1;\nimport 2;\nimport 3;"
    assert str(header_obj) == header


def test_empty_header():
    header = ""
    content = """
# title
"""
    page = header + "\n" + content
    extracted_header = extract_header(page)["header"]
    assert extracted_header == header
    header_obj = Header.from_string(extracted_header)
    assert header_obj.description is None
    assert header_obj.displayed_sidebar is None
    assert header_obj.slug is None
    assert header_obj.imports is None
    assert str(header_obj) == header


def test_header_with_title():
    header = """---
title: Install on on-prem infra
description: Hosting W&B Server on on-premises infrastructure
displayed_sidebar: default
---
import Tabs from '@theme/Tabs';
import TabItem from '@theme/TabItem';"""

    content = """:::info
W&B recommends fully managed deployment options such as [W&B Multi-tenant Cloud](../hosting-options/saas_cloud.md) or [W&B Dedicated Cloud](../hosting-options//dedicated_cloud.md) deployment types. W&B fully managed services are simple and secure to use, with minimum to no configuration required.
:::

You can run W&B Server on your on-premises infrastructure if Multi-tenant Cloud or Dedicated Cloud are not a good fit for your organization."""
    page = header + "\n" + content
    extracted = extract_header(page)
    assert extracted["header"] == header
    header_obj = Header.from_string(extracted["header"])
    assert header_obj.title == "Install on on-prem infra"
    assert header_obj.description == "Hosting W&B Server on on-premises infrastructure"
    assert header_obj.displayed_sidebar == "default"
    assert (
        header_obj.imports
        == "import Tabs from '@theme/Tabs';\nimport TabItem from '@theme/TabItem';"
    )
    assert str(header_obj) == header
    assert extracted["content"] == content


def test_header_serialization_with_japanese_characters():
    header_obj = Header(
        title="オンプレミス インフラストラクチャー",
        description="オンプレミス インフラストラクチャー上での W&B サーバーのホスティング",
        displayed_sidebar="default",
        imports="",
    )
    expected_header = """
---
title: オンプレミス インフラストラクチャー
description: オンプレミス インフラストラクチャー上での W&B サーバーのホスティング
displayed_sidebar: default
---
"""
    assert (
        header_obj.description
        == "オンプレミス インフラストラクチャー上での W&B サーバーのホスティング"
    )
    assert str(header_obj) == expected_header.strip()


def test_header_serialization_with_newlines_in_description():
    header_obj = Header(
        title="Sample Title",
        description="This is a description\nwith multiple lines\nthat should be serialized correctly.\n",
        displayed_sidebar="default",
        imports="",
    )
    expected_header = """
---
title: Sample Title
description: This is a description with multiple lines that should be serialized correctly.
displayed_sidebar: default
---
"""
    assert (
        header_obj.description
        == "This is a description\nwith multiple lines\nthat should be serialized correctly.\n"
    )
    assert str(header_obj) == expected_header.strip()


def test_header_class():
    "Empty header"
    h = Header(
        title=None, description=None, slug=None, displayed_sidebar=None, imports=None
    )
    assert str(h) == ""



from pathlib import Path

import weave

LANGUAGES_DICT = {
    "es": "Spanish",
    "ja": "Japanese",
    "fr": "French",
    "de": "German",
    "ko": "Korean",
    "zh": "Chinese",
}


def filter_dictionary(query, dictionary):
    "Filter out words from the query that are not in the dictionary"
    dictionary = dictionary.split("\n")
    filtered_dict = []
    for line in dictionary:
        dict_word = line.split(":")[0].lower()
        if dict_word in query.lower():
            filtered_dict.append(line)
    return "\n".join(filtered_dict)


class PromptTemplate(weave.Object):
    system_prompt: str  # contains {output_language}, {dictionary}
    human_prompt: str  # contains {md_chunk}
    dictionary: str  # contains dictionary of words and translations
    language: str  # language of the output
    evaluation_prompt: str | None  # contains {original_page} and {translated_page}

    @classmethod
    def from_files(
        cls,
        system_prompt_file,
        human_prompt_file,
        dictionary_file,
        evaluation_prompt_file=None,
    ):
        system_prompt = Path(system_prompt_file).read_text()
        human_prompt = Path(human_prompt_file).read_text()
        dictionary = Path(dictionary_file).read_text()
        language = Path(dictionary_file).stem
        evaluation_prompt = (
            Path(evaluation_prompt_file).read_text() if evaluation_prompt_file else None
        )
        return cls(
            system_prompt=system_prompt,
            human_prompt=human_prompt,
            dictionary=dictionary,
            language=language,
            evaluation_prompt=evaluation_prompt,
        )

    @classmethod
    def from_folder(cls, folder: Path, language: str = "ja"):
        return cls.from_files(
            folder / "system_prompt.txt",
            folder / "human_prompt.txt",
            folder / f"language_dicts/{language}.yaml",
            folder / "evaluation_prompt.txt",
        )

    def __str__(self):
        return (
            f"System Prompt:\n==============\n{self.system_prompt}"
            f"Human Prompt: \n==============\n{self.human_prompt}"
            f"Evaluation Prompt: \n==============\n{self.evaluation_prompt}"
        )

    def filter_dictionary(self, query):
        return filter_dictionary(query, self.dictionary)

    @weave.op
    def format(self, md_chunk):
        messages = [
            {
                "role": "system",
                "content": self.system_prompt.format(
                    output_language=LANGUAGES_DICT[self.language],
                    dictionary=self.filter_dictionary(md_chunk),
                ),
            },
            {
                "role": "user",
                "content": self.human_prompt.format(md_chunk=md_chunk),
            },
        ]
        return messages
